@@ -97,7 +97,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: NmapOps.cc 3274 2006-04-22 23:00:09Z fyodor $ */
+/* $Id: NmapOps.cc 3355 2006-05-15 22:37:31Z fyodor $ */
 #include "nmap.h"
 #include "nbase.h"
 #include "NmapOps.h"
@@ -214,6 +214,7 @@ void NmapOps::Initialize() {
   extra_payload = NULL;
   scan_delay = 0;
   scanflags = -1;
+  defeat_rst_ratelimit = 0;
   resume_ip.s_addr = 0;
   osscan_limit = 0;
   osscan_guess = 0;
@@ -249,6 +250,7 @@ void NmapOps::Initialize() {
   resolve_all = 0;
   dns_servers = NULL;
   noninteractive = false;
+  current_scantype = STYPE_UNKNOWN;
 }
 
 bool NmapOps::TCPScan() {
@@ -415,6 +417,10 @@ void NmapOps::ValidateOptions() {
   if (osscan && pingscan) {
     fatal("WARNING:  OS Scan is unreliable with a ping scan.  You need to use a scan type along with it, such as -sS, -sT, -sF, etc instead of -sP");
   }
+
+  if (defeat_rst_ratelimit && !synscan) {
+      fatal("Option --defeat-rst-ratelimit works only with a SYN scan (-sS)");
+  }
   
   if (resume_ip.s_addr && generate_random_ips)
     resume_ip.s_addr = 0;
