@@ -99,7 +99,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: TargetGroup.cc,v 1.12 2004/08/29 09:12:03 fyodor Exp $ */
+/* $Id: TargetGroup.cc 3391 2006-05-27 08:57:03Z fyodor $ */
 
 #include "TargetGroup.h"
 #include "NmapOps.h"
@@ -179,7 +179,7 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
     if (strchr(hostexp, ':'))
       fatal("Invalid host expression: %s -- colons only allowed in IPv6 addresses, and then you need the -6 switch", hostexp);
 
-    /*strauct in_addr current_in;*/
+    /*struct in_addr current_in;*/
     addy[0] = addy[1] = addy[2] = addy[3] = addy[4] = NULL;
     addy[0] = r = hostexp;
     /* First we break the expression up into the four parts of the IP address
@@ -187,8 +187,8 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
     target_net = strtok(hostexp, "/");
     s = strtok(NULL, "");    /* find the end of the token from hostexp */
     netmask  = ( s ) ? atoi(s) : 32;
-    if ((int) netmask < 0 || netmask > 32) {
-      fprintf(stderr, "Illegal netmask value (%d), must be /0 - /32 .  Assuming /32 (one host)\n", netmask);
+    if ((int) netmask <= 0 || netmask > 32) {
+      fprintf(stderr, "Illegal netmask value (%d), must be /1 - /32 .  Assuming /32 (one host)\n", netmask);
       netmask = 32;
     }
     for(i=0; *(hostexp + i); i++) 
@@ -198,11 +198,18 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
       }
     if (netmask != 32 || namedhost) {
       targets_type = IPV4_NETMASK;
-      if (!inet_aton(target_net, &(startaddr))) {
-	if ((target = gethostbyname(target_net)))
+      if (!inet_pton(AF_INET, target_net, &(startaddr))) {
+	if ((target = gethostbyname(target_net))) {
+          int count=0;
+
 	  memcpy(&(startaddr), target->h_addr_list[0], sizeof(struct in_addr));
-	else {
-	  fprintf(stderr, "Failed to resolve given hostname/IP: %s.  Note that you can't use '/mask' AND '[1-4,7,100-]' style IP ranges\n", target_net);
+
+          while (target->h_addr_list[count]) count++;
+
+          if (count > 1)
+             error("Warning: Hostname %s resolves to %d IPs. Using %s.", target_net, count, inet_ntoa(*((struct in_addr *)target->h_addr_list[0])));
+	} else {
+	  fprintf(stderr, "Failed to resolve given hostname/IP: %s.  Note that you can't use '/mask' AND '1-4,7,100-' style IP ranges\n", target_net);
 	  free(hostexp);
 	  return 1;
 	}
@@ -221,30 +228,24 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
       return 1;
     }
     else {
-      i=0;
       targets_type = IPV4_RANGES;
+      i=0;
+
       while(*++r) {
 	if (*r == '.' && ++i < 4) {
 	  *r = '\0';
 	  addy[i] = r + 1;
 	}
-	else if (*r == '[') {
-	  *r = '\0';
-	  addy[i]++;
-	}
-	else if (*r == ']') *r = '\0';
-	/*else if ((*r == '/' || *r == '\\') && i == 3) {
-	 *r = '\0';
-	 addy[4] = r + 1;
-	 }*/
-	else if (*r != '*' && *r != ',' && *r != '-' && !isdigit((int)*r)) fatal("Invalid character in  host specification.");
+	else if (*r != '*' && *r != ',' && *r != '-' && !isdigit((int)*r)) 
+	  fatal("Invalid character in  host specification.  Note in particular that square brackets [] are no longer allowed.  They were redundant and can simply be removed.");
       }
-      if (i != 3) fatal("Target host specification is illegal.");
+      if (i != 3) fatal("Invalid target host specification: %s", target_expr);
       
       for(i=0; i < 4; i++) {
 	j=0;
-	while((s = strchr(addy[i],','))) {
-	  *s = '\0';
+	do {
+	  s = strchr(addy[i],',');
+	  if (s) *s = '\0';
 	  if (*addy[i] == '*') { start = 0; end = 255; } 
 	  else if (*addy[i] == '-') {
 	    start = 0;
@@ -256,32 +257,17 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
 	    if ((r = strchr(addy[i],'-')) && *(r+1) ) end = atoi(r + 1);
 	    else if (r && !*(r+1)) end = 255;
 	  }
-	  if (o.debugging > 2)
-	    log_write(LOG_STDOUT, "The first host is %d, and the last one is %d\n", start, end);
-	  if (start < 0 || start > end) fatal("Your host specifications are illegal!");
+	  /*	  if (o.debugging > 2)
+		  log_write(LOG_STDOUT, "The first host is %d, and the last one is %d\n", start, end); */
+	  if (start < 0 || start > end || start > 255 || end > 255)
+	    fatal("Your host specifications are illegal!");
+	  if (j + (end - start) > 255) 
+	    fatal("Your host specifications are illegal!");
 	  for(k=start; k <= end; k++)
 	    addresses[i][j++] = k;
-	  addy[i] = s + 1;
-	}
-	if (*addy[i] == '*') { start = 0; end = 255; } 
-	else if (*addy[i] == '-') {
-	  start = 0;
-	  if (!addy[i] + 1) end = 255;
-	  else end = atoi(addy[i]+ 1);
-	}
-	else {
-	  start = end = atoi(addy[i]);
-	  if ((r =  strchr(addy[i],'-')) && *(r+1) ) end = atoi(r+1);
-	  else if (r && !*(r+1)) end = 255;
-	}
-	if (o.debugging > 2)
-	  log_write(LOG_STDOUT, "The first host is %d, and the last one is %d\n", start, end);
-	if (start < 0 || start > end) fatal("Your host specifications are illegal!");
-	if (j + (end - start) > 255) fatal("Your host specifications are illegal!");
-	for(k=start; k <= end; k++) 
-	  addresses[i][j++] = k;
-	last[i] = j - 1;
-	
+	  last[i] = j-1;
+	  if (s) addy[i] = s + 1;
+	} while (s);
       }
     }
   memset((char *)current, 0, sizeof(current));
@@ -302,7 +288,7 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
     hints.ai_family = PF_INET6;
     rc = getaddrinfo(hostexp, NULL, &hints, &result);
     if (rc != 0) {
-      fprintf(stderr, "Failed to resolve given IPv6 hostname/IP: %s.  Note that you can't use '/mask' or '[1-4,7,100-]' style ranges for IPv6.  Error cod %d: %s\n", hostexp, rc, gai_strerror(rc));
+      fprintf(stderr, "Failed to resolve given IPv6 hostname/IP: %s.  Note that you can't use '/mask' or '[1-4,7,100-]' style ranges for IPv6.  Error code %d: %s\n", hostexp, rc, gai_strerror(rc));
       free(hostexp);
       if (result) freeaddrinfo(result);
       return 1;
@@ -325,9 +311,9 @@ int TargetGroup::parse_expr(const char * const target_expr, int af) {
  * get_next_host should be used for skipping the last octet :-) 
  * returns: number of hosts skipped */
 int TargetGroup::skip_range(_octet_nums octet) {
-  int hosts_skipped = 0, /* number of hosts skipped */
-      oct = 0,           /* octect number */
-      i;                 /* simple lcv */
+  unsigned long hosts_skipped = 0, /* number of hosts skipped */
+      oct = 0;           /* octect number */
+      int i = 0;                 /* simple lcv */
 
   /* This function is only supported for RANGES! */
   if (targets_type != IPV4_RANGES)
@@ -351,7 +337,7 @@ int TargetGroup::skip_range(_octet_nums octet) {
   }
 
   /* catch if we try to take more than are left */
-  assert(ipsleft >= hosts_skipped - 1);
+  assert(ipsleft + 1>= hosts_skipped);
 
   /* increment the next octect that we can above us */
   for (i = oct; i >= 0; i--) {
@@ -376,7 +362,7 @@ int TargetGroup::skip_range(_octet_nums octet) {
 }
 
  /* Grab the next host from this expression (if any) and uptdates its internal
-    state to reflect the the IP was given out.  Returns 0 and
+    state to reflect that the IP was given out.  Returns 0 and
     fills in ss if successful.  ss must point to a pre-allocated
     sockaddr_storage structure */
 int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
@@ -390,7 +376,7 @@ int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
   assert(sslen);
 
 
-  if (ipsleft <= 0)
+  if (ipsleft == 0)
     return -1;
   
   if (targets_type == IPV4_NETMASK) {
@@ -437,7 +423,7 @@ int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
       }
     }
     if (octet == -1) {
-      /* It didn't find anything to bump up, I muast have taken the last IP */
+      /* It didn't find anything to bump up, I must have taken the last IP */
       assert(ipsleft == 1);
       /* So I set current to last with the very final octet up one ... */
       /* Note that this may make current[3] == 256 */
@@ -462,7 +448,6 @@ int TargetGroup::get_next_host(struct sockaddr_storage *ss, size_t *sslen) {
 #endif // HAVE_IPV6
   }
   ipsleft--;
-  assert(ipsleft >= 0);
   
   /* If we are resuming from a previous scan, we have already finished
      scans up to o.resume_ip.  */
@@ -508,7 +493,7 @@ int TargetGroup::return_last_host() {
 /* Lookahead is the number of hosts that can be
    checked (such as ping scanned) in advance.  Randomize causes each
    group of up to lookahead hosts to be internally shuffled around.
-   The target_expressions array MUST REMAIN VALID IN MEMMORY as long as
+   The target_expressions array MUST REMAIN VALID IN MEMORY as long as
    this class instance is used -- the array is NOT copied.
  */
 HostGroupState::HostGroupState(int lookahead, int rnd, 
