@@ -3,7 +3,7 @@
  * nbase.h -- The main include file exposing the external API for          *
  * libnbase, a library of base (often compatability) routines.  Programs   *
  * using libnbase can guarantee the availability of functions like         *
- * (v)snprintf and inet_aton.  This library also provides consistency and  *
+ * (v)snprintf and inet_pton.  This library also provides consistency and  *
  * extended features for some functions.  It was originally written for    *
  * use in the Nmap Security Scanner ( http://www.insecure.org/nmap/ ).     *
  *                                                                         *
@@ -101,7 +101,7 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nbase.h 3232 2006-03-26 06:44:57Z fyodor $ */
+/* $Id: nbase.h 3438 2006-06-10 21:23:30Z fyodor $ */
 
 #ifndef NBASE_H
 #define NBASE_H
@@ -112,7 +112,7 @@
  * * 'inline' is defined to what is neccessary for the C compiler being
  *   used (which may be nothing)
  *
- * * snprintf, inet_aton, inet_pton, inet_aton, memcpy, and bzero are 
+ * * snprintf, inet_pton, memcpy, and bzero are 
  *   provided if you don't have them (prototypes for these are 
  *   included either way).
  *
@@ -369,11 +369,6 @@ int gettimeofday(struct timeval *tv, struct timeval *tz);
 unsigned int sleep(unsigned int seconds);
 #endif
 
-#ifndef HAVE_INET_ATON
-struct in_addr;
-int inet_aton(const char *cp, struct in_addr *addr);
-#endif
-
 /* Strncpy is like strcpy() except it ALWAYS zero-terminates, even if
    it must truncate */
 int Strncpy(char *dest, const char *src, size_t n);
