@@ -99,14 +99,17 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: nmap_winconfig.h,v 1.74 2005/02/07 08:52:52 fyodor Exp $ */
+/* $Id: nmap_winconfig.h 3465 2006-06-12 22:24:10Z fyodor $ */
 
 #ifndef NMAP_WINCONFIG_H
 #define NMAP_WINCONFIG_H
-
-#define NMAP_VERSION "3.81"
-#define NMAP_NAME "nmap"
+/* Without this, Windows will give us all sorts of crap about using functions
+   like strcpy() even if they are done safely */
+#define _CRT_SECURE_NO_DEPRECATE 1
+#define NMAP_VERSION "4.10"
+#define NMAP_NAME "Nmap"
 #define NMAP_URL "http://www.insecure.org/nmap"
 #define NMAP_PLATFORM "i686-pc-windows-windows"
-#define NMAPDATADIR "c:\nmap" /* FIXME: I really need to make this dynamic */
+#define NMAPDATADIR "c:\\nmap" /* FIXME: I really need to make this dynamic */
+
 #endif /* NMAP_WINCONFIG_H */
