@@ -97,12 +97,13 @@
  *                                                                         *
  ***************************************************************************/
 
-/* $Id: portlist.h,v 1.23 2004/11/12 20:21:27 fyodor Exp $ */
+/* $Id: portlist.h 3355 2006-05-15 22:37:31Z fyodor $ */
 
 #ifndef PORTLIST_H
 #define PORTLIST_H
 
 #include <nbase.h>
+#include <map>
 
 /* port states */
 #define PORT_UNKNOWN 0
@@ -117,6 +118,8 @@
 #define PORT_HIGHEST_STATE 9 /* ***IMPORTANT -- BUMP THIS UP WHEN STATES ARE 
 				ADDED *** */
  
+#define TCPANDUDP IPPROTO_MAX
+
 #define CONF_NONE 0
 #define CONF_LOW 1
 #define CONF_HIGH 2
@@ -130,6 +133,7 @@ enum serviceprobestate {
   PROBESTATE_FINISHED_SOFTMATCHED, // Well, a soft match anyway
   PROBESTATE_FINISHED_NOMATCH, // D'oh!  Failed to find the service.
   PROBESTATE_FINISHED_TCPWRAPPED, // We think the port is blocked via tcpwrappers
+  PROBESTATE_EXCLUDED, // The port has been excluded from the scan
   PROBESTATE_INCOMPLETE // failed to complete (error, host timeout, etc.)
 };
 
@@ -149,10 +153,13 @@ struct serviceDeductions {
   // confident) expressing how accurate the service detection is
   // likely to be.
   int name_confidence;
-  // Any of these three will be NULL if undetermined.
+  // Any of these 6 can be NULL if we weren't able to determine it
   const char *product;
   const char *version;
   const char *extrainfo;
+  const char *hostname;
+  const char *ostype;
+  const char *devicetype;
   // SERVICE_TUNNEL_NONE or SERVICE_TUNNEL_SSL
   enum service_tunnel_type service_tunnel; 
   // This is the combined version of the three fields above.  It will be 
@@ -179,8 +186,8 @@ class Port {
   Port();
   ~Port();
 
-  // pass in an allocated struct serviceDeductions (don't wory about initializing, and
-  // you don't have to free any inernal ptrs.  See the serviceDeductions definition for
+  // pass in an allocated struct serviceDeductions (don't worry about initializing, and
+  // you don't have to free any internal ptrs.  See the serviceDeductions definition for
   // the fields that are populated.  Returns 0 if at least a name is available.
   int getServiceDeductions(struct serviceDeductions *sd);
 
@@ -197,7 +204,8 @@ class Port {
   // detected and we tried to tunnel through it ).
   void setServiceProbeResults(enum serviceprobestate sres, const char *sname,
 			      enum service_tunnel_type tunnel, const char *product, 
-			      const char *version, 
+			      const char *version, const char *hostname,
+			      const char *ostype, const char *devicetype,
 			      const char *extrainfo, const char *fingerprint);
 
   /* Sets the results of an RPC scan.  if rpc_status is not
@@ -232,16 +240,32 @@ class Port {
   char *serviceprobe_product; 
   char *serviceprobe_version; 
   char *serviceprobe_extrainfo; 
+  char *serviceprobe_hostname;
+  char *serviceprobe_ostype;
+  char *serviceprobe_devicetype;
   enum service_tunnel_type serviceprobe_tunnel;
   // A fingerprint that the user can submit if the service wasn't recognized
   char *serviceprobe_fp;
 
 };
 
+
+/* Needed enums to address some arrays. This values
+ * should never be used directly. Use INPROTO2PORTLISTPROTO macro */
+enum portlist_proto {	// PortList Protocols
+  PORTLIST_PROTO_TCP	= 0,
+  PORTLIST_PROTO_UDP	= 1,
+  PORTLIST_PROTO_IP	= 2,
+  PORTLIST_PROTO_MAX	= 3
+};
+
 class PortList {
  public:
   PortList();
   ~PortList();
+  /* Set ports that will be scanned for each protocol. This function
+   * must be called before any PortList object will be created. */
+  static void initializePortMap(int protocol, u16 *ports, int portcount);
   /* Add a new port to this list.  If the state has changed, it is
      OK to call this function to effect the change */
   int addPort(u16 portno, u8 protocol, char *owner, int state);
@@ -251,36 +275,61 @@ class PortList {
      want).  Only used when printing new port updates.  Optional.  A
      copy is made. */
   void setIdStr(const char *id);
-/* A function for iterating through the ports.  Give NULL for the
+  /* A function for iterating through the ports.  Give NULL for the
    first "afterthisport".  Then supply the most recent returned port
    for each subsequent call.  When no more matching ports remain, NULL
    will be returned.  To restrict returned ports to just one protocol,
-   specify IPPROTO_TCP or IPPROTO_UDP for allowed_protocol.  A 0 for
-   allowed_protocol matches either.  allowed_state works in the same
-   fashion as allowed_protocol. This function returns ports in numeric
+   specify IPPROTO_TCP or IPPROTO_UDP for allowed_protocol. A TCPANDUDP
+   for allowed_protocol matches either. A 0 for allowed_state matches 
+   all possible states. This function returns ports in numeric
    order from lowest to highest, except that if you ask for both TCP &
    UDP, every TCP port will be returned before we start returning UDP
    ports */
-    Port *nextPort(Port *afterthisport, 
-		   u8 allowed_protocol, int allowed_state, 
-		   bool allow_portzero);
+   Port *nextPort(Port *afterthisport, 
+  	          int allowed_protocol, int allowed_state);
+
+  /* Get Port structure from PortList structure.*/
+  Port *getPortEntry(u16 portno, u8 protocol);
+  /* Set Port structure to PortList structure.*/
+  void  setPortEntry(u16 portno, u8 protocol, Port *port);
 
-  Port *lookupPort(u16 portno, u8 protocol);
-  Port **udp_ports;
-  Port **tcp_ports;
-  Port **ip_prots;
-  int state_counts[PORT_HIGHEST_STATE]; /* How many ports in list are in each
-					   state */
-  int state_counts_udp[PORT_HIGHEST_STATE];
-  int state_counts_tcp[PORT_HIGHEST_STATE];
-  int state_counts_ip[PORT_HIGHEST_STATE];
-  int getIgnoredPortState(); /* The state of the port we ignore for output */
   int numports; /* Total number of ports in list in ANY state */
+
+  /* Get number of ports in this state. This a sum for protocols. */
+  int getStateCounts(int state);
+  /* Get number of ports in this state for requested protocol. */
+  int getStateCounts(int protocol, int state);
+
+  /* Cycles through the 0 or more "ignored" ports which should be
+   consolidated for Nmap output.  They are returned sorted by the
+   number of prots in the state, starting with the most common.  It
+   should first be called with PORT_UNKNOWN to obtain the most popular
+   ignored state (if any).  Then call with that state to get the next
+   most popular one.  Returns the state if there is one, but returns
+   PORT_UNKNOWN if there are no (more) states which qualify for
+   consolidation */
+  int nextIgnoredState(int prevstate);
+
+  /* Returns true if a state should be ignored (consolidated), false otherwise */
+  bool isIgnoredState(int state);
+
+  int numIgnoredStates();
+  int numIgnoredPorts();
+
  private:
   /* A string identifying the system these ports are on.  Just used for 
      printing open ports, if it is set with setIdStr() */
   char *idstr;
-
+  /* Number of ports in each state per each protocol */
+  int state_counts_proto[PORTLIST_PROTO_MAX][PORT_HIGHEST_STATE];
+  Port **port_list[PORTLIST_PROTO_MAX];
+ protected:
+  /* Mapps port_number to index in port_list array.
+   * Only functions: getPortEntry, setPortEntry, initializePortMap and 
+   * nextPort should access this structure directly. */
+  static u16 *port_map[PORTLIST_PROTO_MAX];
+  /* Number of allocated elements in port_list per each protocol. */
+  static int port_list_count[PORTLIST_PROTO_MAX];
 };
 
 #endif
